@@ -6,17 +6,19 @@ use strict;
 use 5.010000;
 no warnings 'utf8';
 
-our $VERSION = '0.045';
+our $VERSION = '0.046';
 
 use File::Temp qw( tempfile );
 
-use Clone                  qw( clone );
+#use Clone                  qw( clone );
 use File::Slurp            qw( read_file );
 use List::Util             qw( all );
+#use Spreadsheet::Read      qw( ReadData rows ); # "require"d
 use Term::Choose           qw();
 use Term::Choose::Util     qw( choose_multi );
 use Term::ReadLine::Simple qw();
 use Text::CSV              qw();
+use Text::ParseWords       qw( parse_line );
 
 use App::DBBrowser::Util;
 
@@ -29,7 +31,7 @@ sub new {
 
 
 sub __insert_into {
-    my ( $self, $sql, $table, $qt_columns, $pr_columns, $backup_sql ) = @_;
+    my ( $self, $sql, $table, $qt_columns, $pr_columns ) = @_;
     my $util   = App::DBBrowser::Util->new( $self->{info}, $self->{opt} );
     my $stmt_h = Term::Choose->new( $self->{info}{lyt_stmt_h} );
     my @cols = ( @$pr_columns );
@@ -57,16 +59,13 @@ sub __insert_into {
         }
         my @print_col = map { $choices->[$_] } @idx;
         if ( ! defined $print_col[0] ) {
-            if ( @{$sql->{quote}{chosen_cols}} ) {
-                $sql->{quote}{chosen_cols} = [];
-                $sql->{print}{chosen_cols} = [];
-                @cols = ( @$pr_columns );
-                next COL_NAMES;
-            }
-            else {
-                $sql = clone( $backup_sql );
+            if ( ! @{$sql->{quote}{chosen_cols}} ) {
                 return;
             }
+            $sql->{quote}{chosen_cols} = [];
+            $sql->{print}{chosen_cols} = [];
+            @cols = ( @$pr_columns );
+            next COL_NAMES;
         }
         if ( $print_col[0] eq $self->{info}{ok} ) {
             shift @print_col;
@@ -86,29 +85,30 @@ sub __insert_into {
         }
     }
     my $trs = Term::ReadLine::Simple->new();
-    my $insert_mode;
+    my $previous_file;
 
     VALUES: while ( 1 ) {
-        if ( ! $self->{opt}{insert_mode} ) {
+        my $input_mode;
+        if ( @{$self->{opt}{input_mode}} == 1 ) {
+            $input_mode = $self->{opt}{input_mode}[0];
+        }
+        else {
             $util->__print_sql_statement( $sql, $table, $sql_type );
-            my $choices = [ undef, 'Cols', 'Rows', 'Multirow', 'File' ];
             # Choose
-            $insert_mode = $stmt_h->choose(
-                $choices,
-                { prompt => 'Input mode: ', index => 1 }
+            $input_mode = $stmt_h->choose(
+                [ undef, @{$self->{opt}{input_mode}} ],
+                { prompt => 'Input mode: ' }
             );
-            if ( ! $insert_mode ) {
-                $sql = clone( $backup_sql );
+            if ( ! defined $input_mode ) {
+                $sql->{quote}{chosen_cols} = [];
+                $sql->{print}{chosen_cols} = [];
                 return;
             }
         }
-        else {
-            $insert_mode = $self->{opt}{insert_mode};
-        }
-        if ( $insert_mode <= 2 ) {
+        if ( $input_mode =~ /^(?:Cols|Rows)\z/ ) {
             my ( $last, $add, $del ) = ( '-OK-', 'Add', 'Del' );
             ROWS: while ( 1 ) {
-                if ( $insert_mode == 1 ) {
+                if ( $input_mode eq 'Cols' ) {
                     my $row_idx = @{$sql->{quote}{insert_into_args}};
                     COLS: for my $col_name ( @{$sql->{print}{chosen_cols}} ) {
                         $util->__print_sql_statement( $sql, $table, $sql_type );
@@ -117,17 +117,13 @@ sub __insert_into {
                         push @{$sql->{quote}{insert_into_args}->[$row_idx]}, $col; # show $col immediately in "print_sql_statement"
                     }
                 }
-                elsif ( $insert_mode == 2 ) {
+                elsif ( $input_mode eq 'Rows' ) {
                     my $csv = Text::CSV->new( { map { $_ => $self->{opt}{$_} } @{$self->{info}{csv_opt}} } );
                     $util->__print_sql_statement( $sql, $table, $sql_type );
                     # Readline
                     my $row = $trs->readline( 'Row: ' );
                     if ( ! defined $row ) {
-                        if ( ! @{$sql->{quote}{insert_into_args}} ) {
-                            $sql->{quote}{chosen_cols} = [];
-                            $sql->{print}{chosen_cols} = [];
-                            return;
-                        }
+                        next VALUES if ! @{$sql->{quote}{insert_into_args}};
                         $#{$sql->{quote}{insert_into_args}}--;
                         next ROWS;
                     }
@@ -147,9 +143,7 @@ sub __insert_into {
                     );
                     if ( ! defined $add_row ) {
                         $sql->{quote}{insert_into_args} = [];
-                        $sql->{quote}{chosen_cols}      = [];
-                        $sql->{print}{chosen_cols}      = [];
-                        return;
+                        next VALUES;
                     }
                     elsif ( $add_row eq $last ) {
                         if ( ! @{$sql->{quote}{insert_into_args}} ) {
@@ -159,11 +153,7 @@ sub __insert_into {
                         return;
                     }
                     elsif ( $add_row eq $del ) {
-                        if ( ! @{$sql->{quote}{insert_into_args}} ) {
-                            $sql->{quote}{chosen_cols} = [];
-                            $sql->{print}{chosen_cols} = [];
-                            return;
-                        }
+                        next VALUES if ! @{$sql->{quote}{insert_into_args}};
                         $default = 0;
                         $#{$sql->{quote}{insert_into_args}}--; ###
                         next ASK;
@@ -173,54 +163,187 @@ sub __insert_into {
             }
         }
         else {
-            my $csv = Text::CSV->new( { map { $_ => $self->{opt}{$_} } @{$self->{info}{csv_opt}} } );
-            my $fh;
-            $util->__print_sql_statement( $sql, $table, $sql_type );
-            if ( $insert_mode == 3 ) {
+            my ( $file, $sheet_idx );
+            if ( $input_mode eq 'Multirow' ) {
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 say 'Multirow: ';
                 # STDIN
                 my $input = read_file( \*STDIN );
-                ( $fh ) = tempfile( DIR => $self->{info}{app_dir}, UNLINK => 1 );
+                ( my $fh, $file ) = tempfile( DIR => $self->{info}{app_dir}, SUFFIX => '.csv', UNLINK => 1 );
                 binmode $fh, ':encoding(' . $self->{opt}{encoding_csv_file} . ')';
                 print $fh $input;
                 seek $fh, 0, 0;
-                #$sql->{quote}{insert_into_args} = $csv->getline_all( \*STDIN );
+                if ( $self->{opt}{csv_read} < 2 ) {
+                    $file = $fh;
+                }
+                ( $sql->{quote}{insert_into_args}, $sheet_idx ) = $self->__parse_file( $file );
+                if ( ! @{$sql->{quote}{insert_into_args}} ) {
+                    my $cm = Term::Choose->new( { prompt => 'Press ENTER' } );
+                    $cm->choose( [ 'empty sheet!' ] ); ###
+                    if ( @{$self->{opt}{input_mode}} == 1 ) {
+                        $sql->{quote}{chosen_cols} = [];
+                        $sql->{print}{chosen_cols} = [];
+                        return;
+                    }
+                    next VALUES;
+                }
+                if ( $self->{opt}{row_col_filter} ) {
+                    $self->__filter_input( $sql, $table, $sql_type, $file, $sheet_idx );
+                    if ( ! @{$sql->{quote}{insert_into_args}} ) {
+                        if ( @{$self->{opt}{input_mode}} == 1 ) {
+                            $sql->{quote}{chosen_cols} = [];
+                            $sql->{print}{chosen_cols} = [];
+                            return;
+                        }
+                        next VALUES;
+                    }
+                }
             }
-            elsif ( $insert_mode == 4 ) {
-                # Readline
-                my $file = $trs->readline( 'Path to file: ' );
-                return if ! defined $file;
-                open $fh, '<:encoding(' . $self->{opt}{encoding_csv_file} . ')', $file or die $!;
-                #open my $fh, '<:encoding(' . $self->{opt}{encoding_csv_file} . ')', $file or die $!;
-                #$sql->{quote}{insert_into_args} = $csv->getline_all( $fh );
-                #close $fh;
+            elsif ( $input_mode eq 'File' ) {
+                FILE: while ( 1 ) {
+                    my @files = ();
+                    push @files, $previous_file if defined $previous_file;
+                    my $new_file = 'New file';
+                    if ( @files ) {
+                        $util->__print_sql_statement( $sql, $table, $sql_type );
+                        # Choose
+                        $file = $stmt_h->choose(
+                            [ undef, map( "  $_", @files, $new_file ) ],
+                            { %{$self->{info}{lyt_stmt_v}} }
+                        );
+                        if ( ! defined $file ) {
+                            if ( @{$self->{opt}{input_mode}} == 1 ) {
+                                $sql->{quote}{chosen_cols} = [];
+                                $sql->{print}{chosen_cols} = [];
+                                return;
+                            }
+                            next VALUES;
+                        }
+                        $file =~ s/^\s+//;
+                    }
+                    if ( ! defined $file || $file eq $new_file ) {
+                        $util->__print_sql_statement( $sql, $table, $sql_type );
+                        # Readline
+                        $file = $trs->readline( 'Path to file: ' );
+                        if ( ! defined $file || ! length $file ) {
+                            if ( @{$self->{opt}{input_mode}} == 1 ) {
+                                $sql->{quote}{chosen_cols} = [];
+                                $sql->{print}{chosen_cols} = [];
+                                return;
+                            }
+                            next VALUES;
+                        }
+                    }
+                    $previous_file = $file;
+                    if ( $self->{opt}{csv_read} < 2 && $file =~ /\.csv\z/i ) {
+                        open my $fh, '<:encoding(' . $self->{opt}{encoding_csv_file} . ')', $file or die $!;
+                        $file = $fh;
+                    }
+                    ( $sql->{quote}{insert_into_args}, $sheet_idx ) = $self->__parse_file( $file );
+                    if ( ! defined $sql->{quote}{insert_into_args} ) {
+                        $sql->{quote}{insert_into_args} = [];
+                        next FILE;
+                    }
+                    if ( ! @{$sql->{quote}{insert_into_args}} ) {
+                        my $cm = Term::Choose->new( { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' } );
+                        $cm->choose( [ 'empty file!' ] ); ############
+                        next FILE;
+                    }
+                    if ( $self->{opt}{row_col_filter} ) {
+                        $self->__filter_input( $sql, $table, $sql_type, $file, $sheet_idx );
+                        next FILE if ! @{$sql->{quote}{insert_into_args}};
+                    }
+                    last FILE;
+                }
             }
-            $sql->{quote}{insert_into_args} = $csv->getline_all( $fh );
-            if ( ! @{$sql->{quote}{insert_into_args}} ) {
-                $sql->{quote}{chosen_cols} = [];
-                $sql->{print}{chosen_cols} = [];
-                return;
-            }
-            if ( $self->{opt}{row_col_filter} ) {
-                $self->__filter_input( $sql, $table, $sql_type, $fh );
-            }
-            close $fh;
+            close $file if ref $file eq 'GLOB';
             return;
         }
     }
 }
 
 
+sub __parse_file {
+    my ( $self, $file, $sheet_idx ) = @_;
+    if ( ref $file eq 'GLOB' ) {
+        seek $file, 0, 0;
+        my $tmp = [];
+        if ( $self->{opt}{csv_read} == 0 ) {
+            my $csv = Text::CSV->new( { map { $_ => $self->{opt}{$_} } @{$self->{info}{csv_opt}} } );
+            while ( my $row = $csv->getline( $file ) ) {
+                push @$tmp, $row;
+            }
+        }
+        else {
+            while ( my $row = <$file> ) {
+                chomp $row;
+                push @$tmp, [ parse_line( $self->{opt}{delim}, $self->{opt}{keep}, $row ) ];
+                #push @$tmp, [ split $self->{opt}{delim}, $row ];
+            }
+        }
+        return $tmp;
+    }
+    else {
+        my $cm = Term::Choose->new( { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' } );
+        if ( ! -e $file ) {
+            $cm->choose( [ $file . ' : file not found!' ] );
+            return;
+        }
+        if ( ! -s $file ) {
+            $cm->choose( [ $file . ' : file is empty!' ] );
+            return;
+        }
+        if ( ! -r $file ) {
+            $cm->choose( [ $file . ' : file is not readable!' ] );
+            return;
+        }
+        require Spreadsheet::Read;
+        my $book = Spreadsheet::Read::ReadData( $file, cells => 0, attr => 0, rc => 1, strip => 0 );
+        if ( ! defined $book ) {
+            $cm->choose( [ $file . ' : no book!' ] );
+            return;
+        }
+        if ( $sheet_idx ) {
+            return [ Spreadsheet::Read::rows( $book->[$sheet_idx] ) ], $sheet_idx;
+        }
+        if ( @$book < 2 ) {
+            $cm->choose( [ $file . ' : no sheets!' ] );
+            return;
+        }
+        elsif ( @$book == 2 ) { # first sheet in $book contains meta info
+            $sheet_idx = 1;
+        }
+        else {
+            my @sheets = map { '- ' . ( length $book->[$_]{label} ? $book->[$_]{label} : 'sheet_' . $_ ) } 1 .. $#$book;
+            my $c_sheet = Term::Choose->new();
+            # Choose
+            $sheet_idx = $c_sheet->choose(
+                [ undef, @sheets ],
+                { %{$self->{info}{lyt_stmt_v}}, index => 1, prompt => 'Choose a sheet' }
+            );
+            return if ! defined $sheet_idx;
+            return if $sheet_idx == 0;
+
+        }
+        if ( $book->[$sheet_idx]{maxrow} == 0 ) {
+            my $cm = Term::Choose->new( { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' } );
+            my $sheet = length $book->[$sheet_idx]{label} ? $book->[$sheet_idx]{label} : 'sheet_' . $_;
+            $cm->choose( [ $sheet . ': empty sheet!' ] );
+            return;
+        }
+        return [ Spreadsheet::Read::rows( $book->[$sheet_idx] ) ], $sheet_idx;
+    }
+}
+
+
 sub __filter_input {
-    my ( $self, $sql, $table, $sql_type, $fh ) = @_;
+    my ( $self, $sql, $table, $sql_type, $file, $sheet_idx ) = @_;
     my $util = App::DBBrowser::Util->new( $self->{info}, $self->{opt} );
     my $stmt_h = Term::Choose->new( $self->{info}{lyt_stmt_h} );
-    my $csv = Text::CSV->new( { map { $_ => $self->{opt}{$_} } @{$self->{info}{csv_opt}} } );
     #my $backup = clone $sql->{quote}{insert_into_args};
 
     FILTER: while ( 1 ) {
-        my @pre = ( $self->{info}{ok} );
-        unshift @pre, undef if $self->{opt}{sssc_mode};
+        my @pre = ( undef, $self->{info}{ok} );
         my ( $input_cols, $input_rows_range, $input_rows_choose, $reset ) = ( 'Columns', 'Rows-range', 'Rows-choose', 'Reset' );
         my $choices = [ @pre, $input_cols, $input_rows_range, $input_rows_choose, $reset ];
         $util->__print_sql_statement( $sql, $table, $sql_type );
@@ -231,13 +354,10 @@ sub __filter_input {
         );
         if ( ! defined $choice ) {
             $sql->{quote}{insert_into_args} = [];
-            $sql->{quote}{chosen_cols}      = [];
-            $sql->{print}{chosen_cols}      = [];
             return;
         }
         elsif ( $choice eq $reset ) {
-            seek $fh, 0, 0;
-            $sql->{quote}{insert_into_args} = $csv->getline_all( $fh );
+            ( $sql->{quote}{insert_into_args} ) = $self->__parse_file( $file, $sheet_idx );
             #$sql->{quote}{insert_into_args} = clone $backup;
         }
         elsif ( $choice eq $self->{info}{ok} ) {
@@ -249,9 +369,9 @@ sub __filter_input {
             COLS: while ( 1 ) {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
-                my $choices = [ @pre, map { "col_$_" } 0 .. $#{$sql->{quote}{insert_into_args}[0]} ];
+                my $choices = [ @pre, map { "col_$_" } 1 .. @{$sql->{quote}{insert_into_args}[0]} ];
                 my $prompt = 'Cols: ';
-                $prompt .= join ',', @col_idx if @col_idx;
+                $prompt .= join ',', map { $_ + 1 } @col_idx if @col_idx;
                 $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my @col_nr = $stmt_h->choose(
@@ -271,7 +391,7 @@ sub __filter_input {
                     shift @col_nr;
                     for my $col ( @col_nr ) {
                         $col =~ s/^col_//;
-                        push @col_idx, $col;
+                        push @col_idx, $col - 1;
                     }
                     if ( @col_idx ) {
                         my $tmp = [];
@@ -284,14 +404,15 @@ sub __filter_input {
                 }
                 for my $col ( @col_nr ) {
                     $col =~ s/^col_//;
-                    push @col_idx, $col;
+                    push @col_idx, $col - 1;
                 }
             }
         }
         elsif ( $choice eq $input_rows_range ) {
             my @pre = ();
             unshift @pre, undef if $self->{opt}{sssc_mode};
-            my $choices = [ @pre, map { "@$_" } @{$sql->{quote}{insert_into_args}} ];
+            #my $choices = [ @pre, map { "@$_" } @{$sql->{quote}{insert_into_args}} ];
+            my $choices = [ @pre, map { join ',', @$_ } @{$sql->{quote}{insert_into_args}} ];
             $util->__print_sql_statement( $sql, $table, $sql_type );
             # Choose
             my $first_row = $stmt_h->choose(
@@ -315,8 +436,8 @@ sub __filter_input {
                 $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 $stmt_h->choose(
-                    [ "Last row ($last_row) is less than First row ($first_row)" ],
-                    { %{$self->{info}{lyt_stop}}, prompt => '' }
+                    [ "Last row [$last_row] is less than First row [$first_row]!" ],
+                    { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' }
                 );
                 next FILTER;
             }
